(ns com.aurellem.gb.experience
  (:use (com.aurellem.gb gb-driver util constants))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def experience-pokemon-1 0xD178)

(defn experience-start-address [poke-num]
  (+ experience-pokemon-1
     (* pokemon-record-width poke-num)))

(def experience-record-width 13)

(defn read-experience
  ([^SaveState state poke-num]
     (let [start (experience-start-address poke-num)
           [exp-h
            exp-m
            exp-l
            hp-exp-h
            hp-exp-l
            attack-exp-h
            attack-exp-l
            defense-exp-h
            defense-exp-l
            speed-exp-h
            speed-exp-l
            special-exp-h
            special-exp-l]
           (subvec (vec (memory state))
                   start (+ experience-record-width start))]
           {:main-exp (glue-bytes exp-h exp-m exp-l)
            :hp-exp (glue-bytes hp-exp-h hp-exp-l)
            :attack-exp (glue-bytes attack-exp-h attack-exp-l)
            :defense-exp (glue-bytes defense-exp-h defense-exp-l)
            :speed-exp (glue-bytes speed-exp-h speed-exp-l)
            :special-exp (glue-bytes special-exp-h special-exp-l)}))
     ([poke-num]
     (read-experience @current-state poke-num)))

(defn give-experience
  ([^SaveState state poke-num exp]
     (let [exp* (merge (read-experience state poke-num) exp)
           raw-exp-data
           (flatten
            [(disect-bytes-3 (:main-exp exp*))
             (disect-bytes-2 (:hp-exp exp*))
             (disect-bytes-2 (:attack-exp exp*))
             (disect-bytes-2 (:defense-exp exp*))
             (disect-bytes-2 (:speed-exp exp*))
             (disect-bytes-2 (:special-exp exp*))])]
       (set-memory-range state 
                         (experience-start-address poke-num)
                         raw-exp-data)))
  ([poke-num exp]
     (give-experience @current-state poke-num exp)))
